@@ -5,7 +5,7 @@ rmvESN<-function(n,mu=rep(0,length(lambda)),Sigma=diag(length(lambda)),lambda,ta
   #Validating Lambda
   if(is.null(lambda)){
     #not provided by user
-    stop("Skewness parameter 'lambda' must be provided (zero vector fot the symmetric case).")
+    stop("Skewness parameter 'lambda' must be provided (zero vector for the symmetric case).")
   }else{
     #validate input
     if(length(c(lambda)) != length(c(mu)) | !is.numeric(lambda))stop("Lambda must be numeric and have same dimension than mu.")
@@ -21,17 +21,13 @@ rmvESN<-function(n,mu=rep(0,length(lambda)),Sigma=diag(length(lambda)),lambda,ta
     #validate input
     if(!is.numeric(tau) | length(tau)>1)stop("Tau must be numeric real number.")
     if(tau == 0){
-      return(rmvSN(n,mu,Sigma,lambda))
+      return(rmvSN0(n,mu,Sigma,lambda))
     }
     tautil = tau/sqrt(1+sum(lambda^2))
-    if(tautil < -2.2){
-      if(tautil< -37){
-        #print("normal aproximation")
-        Delta = sqrtm(Sigma)%*%lambda/sqrt(1+sum(lambda^2))
-        return(rmvnorm(n = n,mean = c(mu - tautil*Delta),sigma = Sigma - Delta%*%t(Delta)))
-      }else{
-        stop("Tau parameter too small leading to high rejection. Try incresing tau.")
-      }
+    if(tautil< -37){
+      #print("normal aproximation")
+      Delta = sqrtm(Sigma)%*%lambda/sqrt(1+sum(lambda^2))
+      return(rmvnorm(n = n,mean = c(mu - tautil*Delta),sigma = Sigma - Delta%*%t(Delta)))
     }
     return(rESN0(n = n,mu = mu,Sigma = Sigma,lambda = lambda,tau = tau))
   }
@@ -39,6 +35,7 @@ rmvESN<-function(n,mu=rep(0,length(lambda)),Sigma=diag(length(lambda)),lambda,ta
 
 rESN0<-function(n = 10000,mu=c(0,2),Sigma=diag(2),lambda=c(-1,3),tau=1){
   mu<-as.matrix(mu)
+  Sigma = as.matrix(Sigma)
   lambda<-as.matrix(lambda)
   varphi<-lambda/sqrt(1+sum(lambda^2))
   tautil<-tau/sqrt(1+sum(lambda^2))
@@ -47,24 +44,18 @@ rESN0<-function(n = 10000,mu=c(0,2),Sigma=diag(2),lambda=c(-1,3),tau=1){
   Omega1<- cbind(Sigma,-SS%*%varphi)
   Omega2<- cbind(-t(SS%*%varphi),1)
   Omega<- rbind(Omega1,Omega2)
-  gen <- rmvnorm(n = round(n/max(pnorm(tautil)-0.05,0.02)) + 10,mean = rep(0,p+1),sigma = Omega)
-  bool = gen[,p+1] < tautil
-  nn = sum(bool)
-  gen1 = gen[bool,1:p]
-  muM = matrix(rep(mu,nn),nrow = nn,ncol = p,byrow = TRUE)
-  gen2 = muM + gen1
-  nout = min(n,nrow(gen2))
-  return(gen2[1:nout,])
+  algo = ifelse(pnorm(tautil)< 10^-3,"gibbs","rejection")
+  return(rtmvnorm(n = n,mean = c(mu,0),sigma = Omega,upper = c(rep(Inf,p),tautil),algorithm = algo)[,1:p])
 }
 
-rmvSN = function(n,mu,Sigma,lambda){
-  p = length(lambda)
-  T = abs(rnorm(n))
-  Delta = sqrtm(Sigma)%*%lambda/sqrt(1+sum(lambda^2))
-  Gamma = Sigma - Delta%*%t(Delta)
-  gen  = matrix(NA,n,p)
-  for(i in 1:n){
-    gen[i,] = as.matrix(mu) + T[i]*Delta + sqrtm(Gamma)%*%as.matrix(rnorm(p))
-  }
-  return(gen)
-}
+# rmvSN = function(n,mu,Sigma,lambda){
+#   p = length(lambda)
+#   T = abs(rnorm(n))
+#   Delta = sqrtm(Sigma)%*%lambda/sqrt(1+sum(lambda^2))
+#   Gamma = Sigma - Delta%*%t(Delta)
+#   gen  = matrix(NA,n,p)
+#   for(i in 1:n){
+#     gen[i,] = as.matrix(mu) + T[i]*Delta + sqrtm(Gamma)%*%as.matrix(rnorm(p))
+#   }
+#   return(gen)
+# }
\ No newline at end of file
